function Editor(container_id, options)
{
	//fill container
	var html = "<div class='header'><div class='tools tools-left'></div><div class='tools tools-right'></div></div>";
	html += "<div class='content'><div class='editor-area'><canvas class='graphcanvas' width='1000' height='500' tabindex=0></canvas></div>";
	html += "<div class='node-panel' style='display: none'><div class='node-panel-header'><strong>"+lang.getTranslation("Node info")+"</strong></div><div class='node-panel-list'>";
	html += "</div></div></div>";

	var root = document.createElement("div");
	this.root = root;
	root.className = "litegraph-editor";
	root.innerHTML = html;

	var canvas = root.querySelector(".graphcanvas");

	//create graph
	var graph = this.graph = new LGraph();
	var graphcanvas = this.graphcanvas = new LGraphCanvas(canvas, graph);
	graphcanvas.background_image = "core/imgs/grid.png";
	
	var bgvcanvas = document.createElement("canvas");
	bgvcanvas.width  = 100;		bgvcanvas.height = 100;
	var bgvctx = bgvcanvas.getContext("2d");
	bgvctx.fillStyle = "rgb(62,62,62)";
	bgvctx.rect(0, 0, 100, 100);
	bgvctx.fill();
	bgvctx.strokeStyle = "rgb(50,50,50)";
	for ( var i=0; i<10; i++ )
	{
		bgvctx.beginPath();		bgvctx.moveTo(0.5, i*10+0.5);	bgvctx.lineTo(100.5, i*10+0.5);	bgvctx.stroke();
		bgvctx.beginPath();		bgvctx.moveTo(i*10+0.5,0.5);	bgvctx.lineTo(i*10+0.5,100.5);	bgvctx.stroke();
	}
	bgvctx.strokeStyle = "rgb(40,40,40)";
//	bgvctx.beginPath();		bgvctx.moveTo(0, 0);	bgvctx.lineTo(100, 1);	bgvctx.stroke();
//	bgvctx.beginPath();		bgvctx.moveTo(1, 1);	bgvctx.lineTo(1, 100);	bgvctx.stroke();
	graphcanvas.background_image = "url(" + bgvcanvas.toDataURL() + ")";
	
	graph.onAfterExecute = function () { graphcanvas.draw(true) };
	this.graphcanvas.infopanel = root.querySelector(".node-panel");
	this.graphcanvas.infopanel_list = root.querySelector(".node-panel-list");
	this.graphcanvas.infopanel_visible = (this.graphcanvas.infopanel.style.display === "block");

	//add stuff
	this.addToolsLongpressButton("newsession_button", lang.getTranslation("New"), "", this.bindFunction(this.onNewButton, this), ".tools-left", 700);
	this.addToolsButton("loadsession_button", lang.getTranslation("Load"), "core/imgs/icon-load.png", this.bindFunction(this.onLoadButton, this), ".tools-left");
	this.addToolsButton("savesession_button", lang.getTranslation("Save"), "core/imgs/icon-save.png", this.bindFunction(this.onSaveButton, this), ".tools-left");
	this.addToolsButton("applyobjects_button", lang.getTranslation("Apply"), "core/imgs/icon-apply.png", this.bindFunction(this.onApplyButton, this), ".tools-left");

	this.addToolsButton("playnode_button", lang.getTranslation("Run"), "core/imgs/icon-play.png", this.bindFunction(this.onPlayButton, this), ".tools-right");
	this.addToolsButton("playstepnode_button", lang.getTranslation("Step"), "core/imgs/icon-playstep.png", this.bindFunction(this.onPlayStepButton, this), ".tools-right");
	this.addToolsButton("properties_button", "", "core/imgs/icon-info.png", this.bindFunction(this.onPropertiesButton, this), ".tools-right");

	//append to DOM
	var parent = document.getElementById(container_id);
	if (parent) parent.appendChild(root);

	// graphcanvas.resize();
	graphcanvas.draw(true, true);
}

Editor.prototype.bindFunction = function (func, context)
{
	return function () {	return func.apply(context, arguments);	};
}

Editor.prototype.addToolsButton = function (id, name, icon_url, callback, container)
{
	if (!container) container = ".tools";
	var button = this.createButton(name, icon_url);
	button.id = id;
	button.addEventListener("click", callback); 
	this.root.querySelector(container).appendChild(button);
}

Editor.prototype.addToolsLongpressButton = function (id, name, icon_url, callback, container, timeout )
{
	if (!container) container = ".tools";
	var button = this.createButton(name, icon_url);
	button.id = id;
	button.addEventListener("mousedown", function(){ this.pressTimer = window.setTimeout(callback, timeout)}); 
	button.addEventListener("mouseup", function(){ window.clearTimeout(this.pressTimer)}); 
	
	this.root.querySelector(container).appendChild(button);
}

Editor.prototype.createPanel = function (title, options)
{
	var root = document.createElement("div");
	root.className = "dialog";
	root.innerHTML = "<div class='dialog-header'><span class='dialog-title'>" + title + "</span></div><div class='dialog-content'></div><div class='dialog-footer'></div>";
	root.header = root.querySelector(".dialog-header");
	root.content = root.querySelector(".dialog-content");
	root.footer = root.querySelector(".dialog-footer");
	return root;
}

Editor.prototype.createButton = function (name, icon_url)
{
	var button = document.createElement("button");
	if (icon_url) button.innerHTML = "<img src='" + icon_url + "'/> ";
	button.innerHTML += name;
	return button;
}

Editor.prototype.onNewButton = function ()
{
	if ( graph.status !== LGraph.STATUS_STOPPED ) this.onPlayButton();
	this.graphcanvas.deselectAllNodes();
	this.graph.sendEventToAllNodes("onClear");
	this.graph.configure( "{}", false);
	loadDefaultNodes();
}

Editor.prototype.onLoadButton = function ()
{
	var loadFilePath = moi.filesystem.getOpenFileName(  lang.getTranslation('Open'),  lang.getTranslation('MoI Node editor files')+' (*.nod)|*.nod' );
	if ( !loadFilePath ) return false;
	if ( graph.status !== LGraph.STATUS_STOPPED ) this.onPlayButton();
	this.graphcanvas.deselectAllNodes();
	var data = '', file = moi.filesystem.openFileStream( loadFilePath, 'r' );
	while ( !file.AtEOF ) data += file.readLine();
	file.close();
	this.graph.sendEventToAllNodes("onClear");
	this.graph.configure( JSON.parse(data), false);
}

Editor.prototype.onSaveButton = function ()
{
	var saveFilePath = moi.filesystem.getSaveFileName( lang.getTranslation('Save as')+' ..',  lang.getTranslation('MoI Node editor files')+' (*.nod)|*.nod' );
	if ( !saveFilePath ) return false;
	if ( graph.status !== LGraph.STATUS_STOPPED ) this.onPlayButton();
	var file = moi.filesystem.openFileStream( saveFilePath, 'w' );
	file.writeLine(JSON.stringify(this.graph.serialize()));
	file.close();
}

Editor.prototype.onPlayButton = function ()
{
	var graph = this.graph;
	var button = this.root.querySelector("#playnode_button");

	if (graph.status === LGraph.STATUS_STOPPED) { button.innerHTML = "<img src='core/imgs/icon-stop.png'/> "+lang.getTranslation("Stop"); graph.start(20); }
		else	{ button.innerHTML = "<img src='core/imgs/icon-play.png'/> "+lang.getTranslation("Run"); graph.stop();}
}

Editor.prototype.onPlayStepButton = function ()
{
	this.graph.runStep(1);
	this.graphcanvas.draw(true, true);
}

Editor.prototype.onApplyButton = function ()
{
	this.graph.runStep(1);
	this.graphcanvas.draw(true, true);
	this.graph.sendEventToAllNodes("onApply");
}

Editor.prototype.onPropertiesButton = function ()
{
	if (!this.graphcanvas.infopanel) return;
	this.graphcanvas.infopanel_visible = !this.graphcanvas.infopanel_visible;
	this.graphcanvas.infopanel.style.display = (this.graphcanvas.infopanel_visible) ? "block" : "none";
}

LiteGraph.Editor = Editor;