// MoI Nodeedit
// Basic nodes v.0.80 - Max Smirnov, 2016
// litegraph.js library (c) Javi Agenjo http://tamats.com

(function(){

// Quartz
function Quartz()
{
	this.addInput("Hz","number");
	this.addOutput("Out","number");
	this.properties = { value:0, min:0, max:359, step:1, mode:["Cycled", "Cycled", "Tic-tac"]};
	this.isImmortal = true;
}

Quartz.title = "Quartz";
Quartz.desc = "Quartz clock";

Quartz.prototype.onExecute = function()
{
	this.properties.value += this.properties.step;
	if (this.properties.mode[0] === "Cycled" )
	{
		if ( this.properties.value > this.properties.max ) this.properties.value =  this.properties.min;
		if ( this.properties.value < this.properties.min ) this.properties.value =  this.properties.max;
	}
	else
	{
		if ( this.properties.value > this.properties.max ) { this.properties.value =  this.properties.max; this.properties.step = -this.properties.step; }
		if ( this.properties.value < this.properties.min ) { this.properties.value =  this.properties.min; this.properties.step = -this.properties.step; }
	}
	this.setOutputData(0, this.properties.value);
}

// LiteGraph.registerNodeType("Basic/Quartz", Quartz);

// ######### Constant ##########
function Constant()
{
	this.minsize = [20, 0];
	this.addOutput("value","numarray",{label:"1"});
	this.properties = { value:[1] };
}

Constant.title = "Const";
Constant.desc = "Constant value";

Constant.prototype.onAdded = function()
{
	this.onPropertyChange();
}

Constant.prototype.onPropertyChange = function()
{
	if ( this.properties.value.length > 1) { this.outputs[0].label = "Array"}
	else { this.outputs[0].label = this.properties.value[0].toString() }
	this.computeSize();
};

Constant.prototype.onExecute = function()
{
	this.setOutputData(0, this.properties.value );
}

LiteGraph.registerNodeType("Basic/Constant", Constant);

// ######### Point ##########
function Point()
{
//	this.minsize = [100, 0];
	this.addInput("X","numarray");
	this.addInput("Y","numarray");
	this.addInput("Z","numarray");
	this.addOutput("Point","pointarray");
	this.addOutput("Object","objectlist");
	this.properties = {mode:["Long","Long","Short"], x:[0], y:[0], z:[0]};
}

Point.title = "Point";
Point.desc = "Point";

Point.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, this.properties.x, this.properties.y, this.properties.z );
	this.properties.x = data.inputs[0];
	this.properties.y = data.inputs[1];
	this.properties.z = data.inputs[2];
	this.setOutputData(0, data.outputs[0]);
	this.setOutputData(1, data.outputs[1]);
}

Point.prototype.multiProcess = function(x,y,z)
{
	var out = [];
	out[0] = new pointArray(true, x, y, z);
	out[1] = factory('point', out[0].getPoint(0));
	return out;
}

Point.prototype.onDrawBackground = function(ctx)
{
//	if ( this.properties.x.length > 1) { this.inputs[0].label = "Array"} else { this.inputs[0].label = "X  "+Math.round(100*this.properties.x[0])/100 }
//	if ( this.properties.y.length > 1) { this.inputs[1].label = "Array"} else { this.inputs[1].label = "Y  "+Math.round(100*this.properties.y[0])/100 }
//	if ( this.properties.z.length > 1) { this.inputs[2].label = "Array"} else { this.inputs[2].label = "Z  "+Math.round(100*this.properties.z[0])/100 }
//	this.computeSize();
}

LiteGraph.registerNodeType("Basic/Point", Point);

// #########  Frame  ##########
function Frame()
{
	this.addInput("In","pointarray");
	this.addInput("rX","numarray");
	this.addInput("rY","numarray");
	this.addInput("rZ","numarray");
	this.addOutput("Out","pointarray");
	this.properties = {mode:["Long","Long","Short"], rX:[0], rY:[0], rZ:[0]};
}

Frame.title = "Frame";
Frame.desc = "Frame";

Frame.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, new pointArray(true), this.properties.rX, this.properties.rY, this.properties.rZ);
	this.properties.rX = data.inputs[1];
	this.properties.rY = data.inputs[2];
	this.properties.rZ = data.inputs[3];
	this.setOutputData(0, data.outputs[0]);
}

Frame.prototype.multiProcess = function(f, x, y, z)
{
	f.rotateAll(x, y, z)
	return [f];
}

LiteGraph.registerNodeType("Basic/Frame", Frame);


// ######### MathFunc ##########
function MathFunc()
{
	this.internal = {}
	this.properties = { mode:["Long","Long","Short"], f:"a+b", preset:["a+b", "a+b", "a-b", "a*b", "a/b", "sin[a]", "cos[a]", "pow(a,b)"], a:[1], b:[1]};
}

MathFunc.title = "Math";
MathFunc.desc = "MathFunction";

MathFunc.prototype.onAdded = function()
{
	if ( this.properties.hasOwnProperty("a") && this.inputs.length < 1) this.addInput("a","numarray");
	if ( this.properties.hasOwnProperty("b") && this.inputs.length < 2) this.addInput("b","numarray");
	if ( this.properties.hasOwnProperty("c") && this.inputs.length < 3) this.addInput("c","numarray");
	if ( this.properties.hasOwnProperty("d") && this.inputs.length < 4) this.addInput("d","numarray");
	if (this.outputs.length < 1 ) this.addOutput(this.properties.f, "numarray");
	this.checkFunction();
}

MathFunc.prototype.checkFunction = function()
{
	var valid = true;
	var fnc = this.properties.f;
	fnc = fnc.replace(/rnd/g, "random()");
	fnc = fnc.replace(/\[/g, "(pi/180*(");
	fnc = fnc.replace(/\]/g, "))");
	fnc = fnc.replace(/(?:[a-z$_][a-z0-9$_]*)|(?:[;={}"'!&<>^\\?:])/ig,
		function ($0)
		{
			if (Math.hasOwnProperty($0)) return "Math."+$0;
			else if (Math.hasOwnProperty($0.toUpperCase())) return "Math."+$0.toUpperCase();
			else if ( $0.toLowerCase() === 'a' || $0.toLowerCase() === 'b' || $0.toLowerCase() === 'c' || $0.toLowerCase() === 'd' ) return $0.toLowerCase();
			else	valid = false;
		});
	this.internal.valid = valid;
	this.internal.fnc = fnc;
}

MathFunc.prototype.onPropertyChange = function( property, value )
{
	if ( property === 'preset' ) this.properties['f'] = value;
	this.outputs[0].label = this.properties['f'];
	this.checkFunction();
	this.computeSize();
};

MathFunc.prototype.onGetInputs = function()
{
	var str = "abcd", list = [];
	for ( var i = 0; i<str.length; i++ )
	{
		var inp = str.charAt(i);
		if ( !(inp in this.properties)) { list.push([inp,"numarray", {locked:true}]); break; }
	}
	return list;
}

MathFunc.prototype.onInputAdded = function(inp)
{
	if ( !(inp.name in this.properties)) this.properties[inp.name] = 1;
	for ( var i in this.inputs ) this.inputs[i].locked = true;
	this.inputs[i].locked = false;
	this.setDirtyCanvas(true);
}

MathFunc.prototype.onInputRemoved = function(slot, name)
{
	if ( this.properties[name] ) delete this.properties[name];
	for ( var i in this.inputs ) this.inputs[i].locked = true;
	if ( this.inputs.length > 0 )this.inputs[i].locked = false;
}

MathFunc.prototype.onExecute = function()
{
	var i = this.inputs.length;
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, this.properties.a, this.properties.b, this.properties.c, this.properties.d );

	if ( i>0 ) { this.properties.a = data.inputs[0] }
	if ( i>1 ) { this.properties.b = data.inputs[1] }
	if ( i>2 ) { this.properties.c = data.inputs[2] }
	if ( i>3 ) { this.properties.d = data.inputs[3] }
	this.setOutputData(0, data.outputs[0]);
}

MathFunc.prototype.multiProcess = function(a,b,c,d)
{
	var output;
	if (this.internal.valid)
	{
		try { output = eval(this.internal.fnc) }
		catch (e) { output = 0 }
		if ( output === 'Infinity' ) { output = 99999999 }
		if ( output === '-Infinity' ) { output = -99999999 }
		if ( typeof ( output ) !=='number' ) { output = 0 }
		if ( isNaN( output ) ) { output = 0  }
	}
	else { output = 0 }
	return [output];
}

LiteGraph.registerNodeType("Basic/MathFunc", MathFunc);

// ######### Interp ##########
function Interp()
{
	this.addInput("In","numarray");
	this.addInput("N","numarray");
	this.addOutput("Out","numarray");
	this.properties = { n:[10] };
}

Interp.title = "Interp";
Interp.desc = "Interp";

Interp.prototype.onExecute = function()
{
	var inp = this.getInputData(0, [0]);
	this.properties.n = this.getInputData(1, this.properties.n);
	var n = Math.floor(this.properties.n[0]) - 1;
//	var out = [];
//	var spl = new BS3interp(inp);
//	if ( n === 1 ) { out.push(spl.calcAt(0)); out.push(spl.calcAt(1))}
//	if ( n > 1 )
//	{
//		var step = 1/n;
//		var t = 0;
//		for (var i=0; i<n; i++) {out.push(spl.calcAt(t)); t +=step }
//		out.push(spl.calcAt(1));
//	}
	var out = interpolateArray(inp, n);
	this.setOutputData(0, out);
}

// LiteGraph.registerNodeType("Basic/Interp", Interp);

// ######### Watch ##########
function Watch()
{
	this.size = [60,20];
	this.addInput("value",0,{label:""});
	this.addOutput("value",0,{label:""});
	this.properties = { value:"" };
}

Watch.title = "Watch";
Watch.desc = "Show value of input";

Watch.prototype.onExecute = function()
{
	this.properties.value = this.getInputData(0);
	this.setOutputData(0, this.properties.value);
}

Watch.prototype.onDrawBackground = function(ctx)
{
	//show the current value
	if(this.inputs[0] && this.properties["value"] !== null)	
	{
		if (this.properties["value"].constructor === Number )
			this.inputs[0].label = this.properties["value"].toFixed(3);
		else
		{
			var str = this.properties["value"];
			if(str && str.length) //convert typed to array
				str = Array.prototype.slice.call(str).join(",");
			this.inputs[0].label = str;
		}
	}
}

// LiteGraph.registerNodeType("Basic/watch", Watch);


// ######### MoI Output ##########
function MoIOutput()
{
	this.minsize = [70, 0];
	this.boxcolor = "#F05";
	this.addInput("","objectlist");
	this.properties = {style:["--", "--"]};
	this.sIndex = -1;
	this.tempobjects = moi.geometryDatabase.createObjectList();
}

MoIOutput.title = "Output";
MoIOutput.desc = "Output";

MoIOutput.prototype.onClear = function()
{
	moi.geometryDatabase.removeObjects(this.tempobjects);
	this.tempobjects = moi.geometryDatabase.createObjectList();
}

MoIOutput.prototype.onAdded = function()
{
	if ( !this.properties ) { this.properties = {style:["--", "--"]} } else { if (!this.properties.style) this.properties.style = ["--", "--"]}
	this.properties.style = this.properties.style.slice(0,2);
	var styles = moi.geometryDatabase.getObjectStyles();
	for ( var n=0; n<styles.length; n++ ) this.properties.style.push(styles.item(n).name);
	this.updateStyles();
}

MoIOutput.prototype.updateStyles = function()
{
	var styles = moi.geometryDatabase.getObjectStyles();
	this.sIndex = -1;
	for ( var x=0; x<styles.length; x++ ) if ( this.properties.style[0] === styles.item(x).name ) { this.sIndex = x; break; }
}

MoIOutput.prototype.updateObjects = function()
{
	this.onClear();
	var changeStyle = (this.properties.style[0] !== this.properties.style[1] && this.sIndex !== -1);
	var inObj = this.getInputData(0, moi.geometryDatabase.createObjectList());
	for ( var i = 0; i<inObj.length; i++) this.tempobjects.addObject(changeStyle?inObj.item(i).clone():inObj.item(i));
	for ( var i=this.tempobjects.length; i>0; i-- ) this.tempobjects.item(i-1).setHitTest(0);
	if (changeStyle) this.tempobjects.setProperty( 'styleIndex', this.sIndex);
	moi.geometryDatabase.addObjects(this.tempobjects);
}

MoIOutput.prototype.onPropertyChange = function()
{
	this.updateStyles();
	this.updateObjects();
}

MoIOutput.prototype.onApply = function()
{
	var newobjects = moi.geometryDatabase.createObjectList();
	for ( var i=0; i<this.tempobjects.length; i++) { newobjects.addObject(this.tempobjects.item(i).clone()); newobjects.item(i).styleIndex = this.tempobjects.item(i).styleIndex; }
	moi.geometryDatabase.addObjects(newobjects);
	moi.geometryDatabase.removeObjects(this.tempobjects);
	this.tempobjects = moi.geometryDatabase.createObjectList();
}

MoIOutput.prototype.onExecute = function()
{
	this.updateObjects();
	if(this.inputs[0].link !== null) { if (this.tempobjects.length === 0) {this.boxcolor = "#F80"} else {this.boxcolor = "#0F5"}} else {this.boxcolor = "#F05"}
	this.updateThisNodeGraph();
}

MoIOutput.prototype.onGetCreatedObjects = function()
{
	var list = [];
	for ( var i=0; i<this.tempobjects.length; i++) list.push(this.tempobjects.item(i).id);
	return list;
}

MoIOutput.prototype.getExtraMenuOptions = function(graphcanvas) { var that = this, thatgraph = this.graph; return [{content:lang.getTranslation("Clear"), callback: function() { that.onAdded(); that.onClear(); that.requestUpdate(); }}]; }

MoIOutput.prototype.onDrawForeground = function(ctx)
{
	var title_height = LiteGraph.NODE_TITLE_HEIGHT;
	var old_alpha = ctx.globalAlpha;
	
	ctx.fillStyle = this.bgcolor || LiteGraph.NODE_DEFAULT_BGCOLOR;
	ctx.fillRect(2,-title_height + 2,title_height - 4,title_height - 4);
	ctx.fillRect( 5, 2, this.size[0]-5, this.size[1]-2);
	
	ctx.fillStyle = this.color || LiteGraph.NODE_DEFAULT_COLOR;
	ctx.globalAlpha = 0.5 * old_alpha;
	ctx.fillRect(2,-title_height + 2,title_height - 4,title_height - 4);
	ctx.globalAlpha = old_alpha;

	ctx.fillStyle=this.color || LiteGraph.NODE_DEFAULT_COLOR;
	ctx.fillStyle = this.boxcolor || LiteGraph.NODE_DEFAULT_BOXCOLOR;
	ctx.roundRect(3,-title_height + 3,title_height - 6,title_height - 6, 3);
	ctx.fill();

	var text = (this.tempobjects.length > 0)?this.tempobjects.length.toFixed(0):"--";
	ctx.font = "12px Arial";
	ctx.fillStyle="#AAA";
	ctx.textAlign = "center";
	ctx.fillText(text, this.size[0]/2, this.size[1]/2+4);
	ctx.textAlign = "left";
}

LiteGraph.registerNodeType("Basic/Output", MoIOutput);

})();