// MoI Nodeedit
// Curves v.0.80 - Max Smirnov, 2016
// litegraph.js library (c) Javi Agenjo http://tamats.com

(function(){

// Line
function Line()
{
	this.addInput("a","pointarray");
	this.addInput("b","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short"] };
}

Line.title = "Line";
Line.desc = "Line";

Line.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess );
	this.setOutputData(0, data.outputs[0]);
}

Line.prototype.multiProcess = function(a, b)
{
	return [factory( 'line', a.getPoint(), b.getPoint() )];
}

// Line.prototype.onExecuteOld = function()
// {
// 	var output = moi.geometryDatabase.createObjectList();
// 	var a = this.getInputData(0, new pointArray());
// 	var b = this.getInputData(1, new pointArray());
// 	var lna = a.getLength(), lnb = b.getLength();
// 	var ln = (lna>lnb)?lna:lnb;
// 	if ( lna > 0 && lnb > 0)
// 	{
// 		var astep = lna/ln+1e-12, bstep = lnb/ln+1e-12;
// 		var ac = 0, bc = 0;
// 		for ( var p = 0; p<ln; p++)
// 		{
// 			var o = factory('line', a.getPoint(~~ac), b.getPoint(~~bc));
// 			if (o.length>0) output.addObject(o.item(0));
// 			ac += astep;
// 			bc += bstep;
// 		}
// 	}
// 	this.setOutputData(0, output);
// }

LiteGraph.registerNodeType("Curves/Line", Line);

// Circle
function Circle()
{
	this.addInput("Center","pointarray");
	this.addInput("Radius","numarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short"], radius:[1] };
}

Circle.title = "Circle";
Circle.desc = "Circle";

Circle.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, this.properties.radius );
	this.properties.radius = data.inputs[1];
	this.setOutputData(0, data.outputs[0]);
}

Circle.prototype.multiProcess = function(center, radius)
{
	return [factory( 'circle', true, center.getFrame() , null, radius )];
}

LiteGraph.registerNodeType("Curves/Circle", Circle);

// Rectangle
function Rectangle()
{
//	this.size = [60,20];
	this.addInput("Center","pointarray");
	this.addInput("Width","numarray");
	this.addInput("Height","numarray");
	this.addInput("Corners","numarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short"], width:[10], height:[10], corners:[0] };
}

Rectangle.title = "Rectangle";
Rectangle.desc = "Rectangle";

Rectangle.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, this.properties.width, this.properties.height, this.properties.corners );
	this.properties.width = data.inputs[1];
	this.properties.height = data.inputs[2];
	this.properties.corners = data.inputs[3];
	this.setOutputData(0, data.outputs[0]);
}

Rectangle.prototype.multiProcess = function(center, w, h, c)
{
	if ( c > 0 ) { return [factory( 'rectcenter', center.getFrame(), null, w, h, true, null, c) ] }
	else { return [factory( 'rectcenter', center.getFrame(), null, w, h, false) ] }
}

LiteGraph.registerNodeType("Curves/Rectangle", Rectangle);

// Polygon
function Polygon()
{
	this.addInput("Center","pointarray");
	this.addInput("Radius","numarray");
	this.addInput("Sides","numarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short"], radius:[1], sides:[6], type:["Circumscribed","Circumscribed","Inscribed"] };
}

Polygon.title = "Polygon";
Polygon.desc = "Polygon";

Polygon.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, this.properties.radius, this.properties.sides);
	this.properties.radius = data.inputs[1];
	this.properties.sides = data.inputs[2];
	this.setOutputData(0, data.outputs[0]);
}

Polygon.prototype.multiProcess = function(center, r, s)
{
	var frame = center.getFrame()
	return [factory( 'polygon', frame, frame.evaluate( 0, r, 0 ), s, (this.properties.type[0] === "Circumscribed"))];
}

LiteGraph.registerNodeType("Curves/Polygon", Polygon);

// Star
function Star()
{
	this.addInput("Center","pointarray");
	this.addInput("Radius","numarray");
	this.addInput("Ratio","numarray");
	this.addInput("Sides","numarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short"], radius:[1], ratio:[0.4], sides:[5] };
}

Star.title = "Star";
Star.desc = "Star";

Star.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, this.properties.radius, this.properties.ratio, this.properties.sides);
	this.properties.radius = data.inputs[1];
	this.properties.ratio = data.inputs[2];
	this.properties.sides = data.inputs[3];
	this.setOutputData(0, data.outputs[0]);
}

Star.prototype.multiProcess = function(center, r, rt, s)
{
	var frame = center.getFrame();
	return [factory( 'polygonstar', frame, frame.evaluate( 0, r, 0 ), s, frame.evaluate( 0, r*rt, 0 ))];
}

LiteGraph.registerNodeType("Curves/Star", Star);

// Curve
function Curve()
{
	this.addInput("In","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Curve", "Curve", "Interpcurve"]};
}

Curve.title = "Curve";
Curve.desc = "Curve";

Curve.prototype.onExecute = function()
{
	var input = this.getInputData(0, new pointArray()), inLen = input.getLength(), factory;
	
	if (this.properties.mode[0] === "Curve") { factory = moi.command.createFactory( 'curve' ) }
	else if (this.properties.mode[0] === "Interpcurve") { factory = moi.command.createFactory( 'interpcurve' ) }
	for (var i = 0; i<inLen; i++)
	{
		factory.createInput('point');
		factory.setInput(i, input.getPoint(i));
	}
	var output = factory.calculate();
	factory.cancel();
	this.setOutputData(0, output);
}
LiteGraph.registerNodeType("Curves/Curve", Curve);

// Polyline
function Polyline()
{
	this.addInput("In","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = {};
}

Polyline.title = "Polyline";
Polyline.desc = "Polyline";

Polyline.prototype.onExecute = function()
{
	var input = this.getInputData(0, new pointArray()), inLen = input.getLength(), factory;
	
	factory = moi.command.createFactory( 'polyline' );
	for (var i = 0; i<inLen; i++)
	{
		factory.createInput('point');
		factory.setInput(i, input.getPoint(i));
	}
	var output = factory.calculate();
	factory.cancel();
	this.setOutputData(0, output);
}
LiteGraph.registerNodeType("Curves/Polyline", Polyline);

})();