// MoI Nodeedit
// Widgets v.0.81 - Max Smirnov, 2016
// litegraph.js library (c) Javi Agenjo http://tamats.com

(function(){
	
	
// -------- Widget Knob --------------------------------------------------------------------------------------------------------------------------------------

function WidgetKnob()
{
	this.size = [55,58];
	this.minsize = [55,58];
	this.addOutput("",'numarray');
	this.local = { csize:50 }
	this.properties = { min:0, max:100, digits:0, value:"20" };
	this.internal = { position:0.2 };
}

WidgetKnob.title = "Knob";
WidgetKnob.desc = "Circular controller";

WidgetKnob.prototype.onAdded = function()
{
	this.internal.position = (this.properties.value - this.properties.min) / (this.properties.max - this.properties.min);
}

WidgetKnob.prototype.onPropertyChange = function()
{

	this.properties.value = parseFloat(this.properties.value.replace(',', '.'));
	if ( isNaN(this.properties.value) ) this.properties.value = this.properties.min;
	if ( this.properties.min >= this.properties.max ) this.properties.max = this.properties.min+1;
	if ( this.properties.value < this.properties.min ) this.properties.value = this.properties.min*1;
	if ( this.properties.value > this.properties.max ) this.properties.value =this.properties.max*1;
	this.properties.digits = Math.floor(this.properties.digits);
	if (this.properties.digits>12) this.properties.digits = 12;
	if (this.properties.digits<0) this.properties.digits = 0;
	this.internal.position = (this.properties.value-this.properties.min)/(this.properties.max-this.properties.min);
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
}

WidgetKnob.prototype.onDrawForeground = function(ctx)
{
	var cx = this.size[0]/2;
	var cy = this.size[1]/2;
	var ro = this.local.csize*0.45;
	var ri = this.local.csize*0.37;

	var bgr = this.bgcolor || LiteGraph.NODE_DEFAULT_BGCOLOR;
	ctx.fillStyle= bgr;
	ctx.beginPath();
		ctx.arc(cx, cy, this.local.csize*0.54, 0, Math.PI*2, false);
	ctx.fill();
	ctx.beginPath();
		ctx.strokeStyle= this.mouseOver ? "#FFF" : "#AAA";
		ctx.arc(this.size[0]/2, this.size[1]/2, this.local.csize*0.50, 0, 2*Math.PI, true);
	ctx.stroke();

	if(this.internal.position > 0)
	{
		ctx.beginPath();
			ctx.fillStyle="#9CA";
			ctx.moveTo(cx,cy-ro)
			ctx.arc(cx, cy, ro, 3/4*Math.PI*2, 3/4*Math.PI*2-Math.PI*2*this.internal.position, true);
			ctx.lineTo(cx,cy);
		ctx.fill();
		ctx.beginPath();
			ctx.fillStyle= bgr;
			ctx.arc(cx, cy, ri, 0, Math.PI*2, false);
		ctx.fill();

	}
	ctx.font = "12px Arial";
	ctx.fillStyle="#AAA";
	ctx.textAlign = "center";
	var dgt = parseInt(this.properties.digits);
	if ( dgt > 2 ) dgt = 2;
	if ( this.properties.max >= 100 && dgt > 1) dgt = 1;
	if ( this.properties.max >= 1000 ) dgt = 0;
	ctx.fillText(parseFloat(this.properties.value).toFixed(dgt), this.size[0]/2, this.size[1]/2+4 );
	ctx.textAlign = "left";
}

WidgetKnob.prototype.onExecute = function()
{
	this.setOutputData(0, [parseFloat(this.properties.value)] );
}

WidgetKnob.prototype.onMouseDown = function(e, skipcapture)
{
	this.center = [this.size[0]/2, this.size[1]/2];
	this.radius = this.size[0]/2;
	if(e.canvasY - this.pos[1] < 0 || distance([e.canvasX,e.canvasY],[this.pos[0] + this.center[0],this.pos[1] + this.center[1]]) > this.radius) return false;
	if ( skipcapture ) return true;
	this.oldmouse = [ e.canvasX - this.pos[0], e.canvasY - this.pos[1] ];
	this.captureInput(true);
	return true;
}

WidgetKnob.prototype.onMouseMove = function(e)
{
	if (!this.oldmouse) return;

	var m = [ e.canvasX - this.pos[0], e.canvasY - this.pos[1] ];

	var v = this.internal.position;
	var delta = (m[1] - this.oldmouse[1]);

	v -= delta/201;
	if (v > 1.0) { v = 1.0 } else if (v < 0.0) { v = 0.0 }

	this.internal.position = v;
	this.properties.value = this.properties.min + (this.properties.max - this.properties.min) * this.internal.position;
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
	this.oldmouse = m;
	this.updateThisNodeGraph();
	this.graph.setisChangedFlag(this.id);
}

WidgetKnob.prototype.onMouseUp = function()
{
	if(this.oldmouse)
	{
		this.oldmouse = null;
		this.captureInput(false);
	}
}

LiteGraph.registerNodeType("Widget/Knob", WidgetKnob);


// ------ Widget H SLIDER -----------------------------------------------------------------------------------------------------------------------------------------------

function WidgetHSlider()
{
	this.size = [150,20];
	this.minsize = [150,20];
	this.addOutput("",'numarray');
	this.local = { shiftLeft:14, shiftRight:40, fontsize:12 };
	this.properties = { min:0, max:10, digits:1, value:"5.0" };
	this.internal = { position:0.5 };
}

WidgetHSlider.title = "Slider";
WidgetHSlider.desc = "Linear slider controller";

WidgetHSlider.prototype.onAdded = function()
{
	this.internal.position = (this.properties.value - this.properties.min) / (this.properties.max - this.properties.min);
}

WidgetHSlider.prototype.onPropertyChange = function()
{
	this.properties.value = parseFloat(this.properties.value.replace(',', '.'));
	if ( isNaN(this.properties.value) ) this.properties.value = this.properties.min;
	if ( this.properties.min >= this.properties.max ) this.properties.max = this.properties.min+1;
	if ( this.properties.value < this.properties.min ) this.properties.value = this.properties.min;
	if ( this.properties.value > this.properties.max ) this.properties.value = this.properties.max;
	this.properties.digits = Math.floor(this.properties.digits);
	if (this.properties.digits>12) this.properties.digits = 12;
	if (this.properties.digits<0) this.properties.digits = 0;
	this.internal.position = (this.properties.value-this.properties.min)/(this.properties.max-this.properties.min);
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
}

WidgetHSlider.prototype.onDrawForeground = function(ctx)
{
	var sr = this.local.shiftRight, sl = this.local.shiftLeft;
	var dgt = parseInt(this.properties.digits);
	if ( dgt > 2 ) dgt = 2;
	if ( this.properties.max >= 100 && dgt > 1) dgt = 1;
	if ( this.properties.max >= 1000 ) dgt = 0;

//	ctx.lineWidth = 1;
	ctx.fillStyle=this.bgcolor || LiteGraph.NODE_DEFAULT_BGCOLOR;
	ctx.fillRect( 3, 2, this.size[0]-6.5, 16);
	
	ctx.fillStyle="#222";
	ctx.fillRect( sl, 8, this.size[0]-sr-sl, 4);
	ctx.beginPath();
		ctx.arc(this.size[0]-sr, 10, 2, 0, 2 * Math.PI, false);
	ctx.fill();

	ctx.fillStyle="#888";
	ctx.fillRect( sl, 8, (this.size[0]-sr-sl)*this.internal.position, 4);
	ctx.beginPath();
		ctx.arc(sl, 10, 2, 0, 2 * Math.PI, false);
	ctx.fill();

	ctx.fillStyle="#AAA";
	ctx.beginPath();
		ctx.arc(sl+(this.size[0]-sr-sl)*this.internal.position, 10, 6, 0, 2 * Math.PI, false);
	ctx.fill();

	ctx.fillStyle="#AAA";
	ctx.font = (this.local.fontsize) + "px Arial";
	ctx.textAlign = "center";
	ctx.fillText(parseFloat(this.properties.value).toFixed(dgt), this.size[0]-sr+20, (10+this.local.fontsize/3));

}

WidgetHSlider.prototype.onExecute = function()
{
	this.setOutputData(0, [parseFloat(this.properties.value)] );
}

WidgetHSlider.prototype.onMouseDown = function(e)
{
	if ( e.canvasX < this.pos[0]+this.local.shiftLeft || e.canvasX > this.pos[0]+this.size[0]-this.local.shiftRight ) return false;
	if (e.canvasY - this.pos[1] < 0) return false;
	this.capture = true;
	this.captureInput(true);
	this.onMouseMove(e);
	return true;
}

WidgetHSlider.prototype.onMouseMove = function(e)
{
	if (!this.capture) return;
	var v = (e.canvasX - this.pos[0] - this.local.shiftLeft)/(this.size[0]-this.local.shiftRight-this.local.shiftLeft);
	if ( v < 0 ) { v = 0 } else if ( v > 1 ) { v = 1 }
	this.internal.position = v;
	this.properties.value = this.properties.min + (this.properties.max - this.properties.min) * this.internal.position;
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
	this.updateThisNodeGraph();
	this.graph.setisChangedFlag(this.id);
}

WidgetHSlider.prototype.onMouseUp = function()
{
	this.capture = false;
	this.captureInput(false);
}

LiteGraph.registerNodeType("Widget/Slider", WidgetHSlider );

// -----------------------------------------------------------------------------------------------------------------------------------------------------
})();