// MoI Nodeedit
// Transform v.0.80 - Max Smirnov, 2016
// litegraph.js library (c) Javi Agenjo http://tamats.com

(function(){
	
// Move
function Move()
{
	this.addInput("Objects","objectlist");
	this.addInput("dX","numarray");
	this.addInput("dY","numarray");
	this.addInput("dZ","numarray");
	this.addOutput("Out","objectlist");
	this.properties = {mode:["Long","Long","Long+","Short","Short+"], dx:[0], dy:[0], dz:[0]};
}

Move.title = "Move";
Move.desc = "Move objects";

Move.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, this.properties.dx, this.properties.dy, this.properties.dz );
	this.properties.dx = data.inputs[1];
	this.properties.dy = data.inputs[2];
	this.properties.dz = data.inputs[3];
	this.setOutputData(0, data.outputs[0]);
}

Move.prototype.multiProcess = function(o, dx, dy, dz)
{
	return [factory('move', o, moi.VectorMath.createPoint(0,0,0), moi.VectorMath.createPoint(dx, dy , dz), true )];
}

LiteGraph.registerNodeType("Transform/Move", Move);

// Orient
function Orient()
{
	this.addInput("Objects","objectlist");
	this.addInput("From","pointarray");
	this.addInput("To","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = {mode:["Long","Long","Long+","Short","Short+"]};
}

Orient.title = "Orient";
Orient.desc = "Orient objects";

Orient.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess );
	this.setOutputData(0, data.outputs[0]);
}

Orient.prototype.multiProcess = function(o, f, t)
{
	return [factory('orient', o, f.getFrame(0), t.getFrame(0), true )];
}

LiteGraph.registerNodeType("Transform/Orient", Orient);

// Object Rotate
function ObjectRotate()
{
	this.addInput("Objects","objectlist");
	this.addInput("Frame","pointarray");
	this.addInput("rX","numarray");
	this.addInput("rY","numarray");
	this.addInput("rZ","numarray");
	this.addOutput("Out","objectlist");
	this.properties = {mode:["Long","Long","Long+","Short","Short+"], rX:[0], rY:[0], rZ:[0]};
}

ObjectRotate.title = "Rotate";
ObjectRotate.desc = "Rotate objects";

ObjectRotate.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, null, this.properties.rX, this.properties.rY, this.properties.rZ );
	this.properties.rX = data.inputs[2];
	this.properties.rY = data.inputs[3];
	this.properties.rZ = data.inputs[4];
	this.setOutputData(0, data.outputs[0]);
}

ObjectRotate.prototype.multiProcess = function(o, f, x, y, z)
{
	return [factory('orient', o, f.getFrame(0), f.getFrame(0, x, y, z), true )];
}

LiteGraph.registerNodeType("Transform/Rotate", ObjectRotate);

// CircularArray
function CircularArray()
{
	this.addInput("Objects","objectlist");
	this.addInput("Center","pointarray");
	this.addInput("Count","numarray");
	this.addOutput("Out","objectlist");
	this.properties = { count:[6] };
}

CircularArray.title = "Circular Array";
CircularArray.desc = "Circular Array";

CircularArray.prototype.onExecute = function()
{
	var inObj = this.getInputData(0, moi.geometryDatabase.createObjectList());
	var c = this.getInputData(1, new pointArray(true));
	this.properties.count[0] = this.getInputData(2, this.properties.count[0]);
	this.properties.count[0] = Math.floor(this.properties.count[0]);
	var outObj = moi.geometryDatabase.createObjectList();
	if ( inObj.length > 0 && this.properties.count[0] > 0 )
	{
		outObj = factory( 'arraycircular', inObj, c.getFrame(0), this.properties.count[0], 360 );
		for ( var i = 0; i<inObj.length; i++ ) outObj.addObject(inObj.item(i));
	}
	this.setOutputData(0, outObj);
}

LiteGraph.registerNodeType("Transform/CircularArray", CircularArray);


})();